@@ -1469,13 +1469,32 @@ static SignalHandler *RegisterMagickSignalHandler(int signal_number)
   return(handler);
 }
 
+static void SetClientNameAndPath(const char *path)
+{
+  char
+    execution_path[MagickPathExtent],
+    filename[MagickPathExtent];
+
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
+  if ((path != (const char *) NULL) && (IsPathAccessible(path) != MagickFalse))
+#else
+  if ((path != (const char *) NULL) && (*path == *DirectorySeparator) &&
+      (IsPathAccessible(path) != MagickFalse))
+#endif
+    (void) CopyMagickString(execution_path,path,MagickPathExtent);
+  else
+    (void) GetExecutionPath(execution_path,MagickPathExtent);
+  GetPathComponent(execution_path,TailPath,filename);
+  (void) SetClientName(filename);
+  GetPathComponent(execution_path,HeadPath,execution_path);
+  (void) SetClientPath(execution_path);
+}
+
 MagickExport void MagickCoreGenesis(const char *path,
   const MagickBooleanType establish_signal_handlers)
 {
   char
-    *events,
-    execution_path[MagickPathExtent],
-    filename[MagickPathExtent];
+    *events;
 
   /*
     Initialize the Magick environment.
@@ -1494,6 +1513,7 @@ MagickExport void MagickCoreGenesis(const char *path,
     }
   (void) SemaphoreComponentGenesis();
   (void) ExceptionComponentGenesis();
+  SetClientNameAndPath(path);
   (void) LogComponentGenesis();
   (void) LocaleComponentGenesis();
   (void) RandomComponentGenesis();
@@ -1506,22 +1526,6 @@ MagickExport void MagickCoreGenesis(const char *path,
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
   NTWindowsGenesis();
 #endif
-  /*
-    Set client name and execution path.
-  */
-#if defined(MAGICKCORE_WINDOWS_SUPPORT)
-  if ((path != (const char *) NULL) && (IsPathAccessible(path) != MagickFalse))
-#else
-  if ((path != (const char *) NULL) && (*path == *DirectorySeparator) &&
-      (IsPathAccessible(path) != MagickFalse))
-#endif
-    (void) CopyMagickString(execution_path,path,MagickPathExtent);
-  else
-    (void) GetExecutionPath(execution_path,MagickPathExtent);
-  GetPathComponent(execution_path,TailPath,filename);
-  (void) SetClientName(filename);
-  GetPathComponent(execution_path,HeadPath,execution_path);
-  (void) SetClientPath(execution_path);
   if (establish_signal_handlers != MagickFalse)
     {
       /*
@@ -1618,8 +1622,13 @@ MagickExport void MagickCoreGenesis(const char *path,
 */
 MagickExport void MagickCoreTerminus(void)
 {
+  InitializeMagickMutex();
+  LockMagickMutex();
   if (magickcore_instantiated == MagickFalse)
-    return;
+    {
+      UnlockMagickMutex();
+      return;
+    }
   MonitorComponentTerminus();
   RegistryComponentTerminus();
 #if defined(MAGICKCORE_X11_DELEGATE)
@@ -1656,8 +1665,9 @@ MagickExport void MagickCoreTerminus(void)
   LocaleComponentTerminus();
   LogComponentTerminus();
   ExceptionComponentTerminus();
-  SemaphoreComponentTerminus();
   magickcore_instantiated=MagickFalse;
+  UnlockMagickMutex();
+  SemaphoreComponentTerminus();
 }
 
 /*
